//
//  EMBAnnotationTreeProvider.h
//  EmberCommon
//
//  Created by Damien DeVille on 12/04/2013.
//  Copyright (c) 2013 Realmac Software. All rights reserved.
//

#import <Foundation/Foundation.h>
#import <QuartzCore/QuartzCore.h>
#import <ImageIO/ImageIO.h>

#import "EMBEmberAnnotations/EMBEmberAnnotations-Constants.h"

@class EMBAnnotation;

@interface EMBAnnotationTreeProvider : NSObject

/*!
	\brief
	Create a new annotation tree provider.
	
	\param snapImageSource
	An image source for the main snap image.
	
	\param snapImagePixelSize
	The size of the main snap image in pixels.
	
	\param snapImageOrientation
	The orientation of the main snap image.
	
	\param snapImageCropArea
	The crop area for the snap image in pixels.
	
	\params annotations
	An array of annotations to display on the snap image.
 */
- (id)initWithSnapImageSource:(CGImageSourceRef)snapImageSource snapImagePixelSize:(CGSize)snapImagePixelSize snapImageOrientation:(EMBEmberAnnotationOrientation)snapImageOrientation snapImageCropArea:(CGRect)snapImageCropArea annotations:(NSArray *)annotations;

/*!
	\brief
	The orientation of the snap image.
 */
@property (assign, nonatomic) EMBEmberAnnotationOrientation snapImageOrientation;

/*!
	\brief
	The crop area for the snap image in pixels.
 */
@property (assign, nonatomic) CGRect snapImageCropArea;

/*!
	\brief
	All the annotations in the tree.
 */
@property (strong, nonatomic) NSArray *annotations;

/*!
	\brief
	Refresh each layer making sure it matches the annotation model.
 */
- (void)refreshLayers;

/*!
	\brief
	Refresh the image layer from contents. You might need to call this method when the screen resolution changes.
 */
- (void)refreshImageLayerContents;

/*!
	\brief
	Set each annotation as dirty and perform a refresh
 */
- (void)reloadLayers;

/*!
	\brief
	The layer created by the layer tree provider. This layer will contain a tree of annotations.
 */
@property (readonly, strong, nonatomic) CALayer *layer;

/*!
	\brief
	Update each layer size and position in the layer tree.
	
	\param size
	The size represents the target size for the root layer.
 */
- (void)updateLayerTreeForLayerTargetSize:(CGSize)size;

/*!
	\brief
	Find the deepest annotation in the tree under a certain point.
	
	\param point
	The location to hit test under. It has to be in the coordinates of the root layer's superlayer.
 */
- (EMBAnnotation *)hitTestAnnotation:(CGPoint)point;

/*!
	\brief
	Update the position of an annotation in the layer tree.
	
	\param annotation
	The annotation for which the position has to be updated.
	
	\param layerPosition
	The new position of the annotation in the tree. It has to be in the coordinates system of the snap image, regardless of any crop.
 */
- (void)updateAnnotation:(EMBAnnotation *)annotation layerPosition:(CGPoint)layerPosition;

/*!
	\brief
	Turn on and off the visibility of the annotations.
	Default to YES.
 */
@property (assign, getter = areAnnotationsVisible, nonatomic) BOOL annotationsVisible;

/*!
	\brief
	Show or hide a specific annotation in the tree.
	
	\param annotation
	The annotation for which the visibility has to be updated.
	
	\param visible
	The visibility of the annotation.
 */
- (void)updateAnnotation:(EMBAnnotation *)annotation visible:(BOOL)visible;

/*!
	\brief
	The bounding box of an annotation, in model coordinates, as positioned in the tree, taking into accoung cropping and orientation.
	
	\param annotation
	An annotation that is currently positioned in the annotation tree.
 */
- (CGRect)boundingBoxForAnnotation:(EMBAnnotation *)annotation;

@end

@interface EMBAnnotationTreeProvider (Rendering)

- (void)renderInContext:(CGContextRef)context;

@end
