//
//  RMPersistentStoreLoader.h
//  BugReporter
//
//  Created by Damien DeVille on 26/03/2012.
//  Copyright (c) 2012 Realmac Software. All rights reserved.
//

#import <Foundation/Foundation.h>

@class NSManagedObjectModel, NSPersistentStore, NSPersistentStoreCoordinator;

@interface EMBPersistentStoreLoader : NSObject

/*!
	\brief
	Create a managed object model merging all the data model files present in the main bundle.
	
	Think of it more as a convenience initializer than a singleton, even if it essentially is a singleton.
 */
+ (id)defaultLoader;

- (id)initWithManagedObjectModelURL:(NSURL *)managedObjectModelURL;
- (id)initWithManagedObjectModel:(NSManagedObjectModel *)managedObjectModel;	// designated initializer

@property (readonly, strong, nonatomic) NSManagedObjectModel *managedObjectModel;
@property (readonly, strong, nonatomic) NSPersistentStoreCoordinator *persistentStoreCoordinator;

/*!
	\brief
	The type of the completion handler called back when a store adding finished.
	
	In case persistentStore references a valid object, addingError is undefined.
 */
typedef void (^EMBPersistentStoreLoaderAddingCompletionHandler)(NSPersistentStore *persistentStore, NSError *addingError);

/*!
	\brief
	Asynchronously add the default persistent store.
	
	Will use the default location, in the Application Support folder, in a folder named after the executable name. Will use the SQLite type and no configuration.
	Will set the following options to YES: NSMigratePersistentStoresAutomaticallyOption, NSInferMappingModelAutomaticallyOption.
	
	If no queue is specified, the completion handler block will be invoked on the main queue.
 */
- (void)addDefaultLocalPersistentStoreWithHandlerQueue:(NSOperationQueue *)handlerQueue completionHandler:(EMBPersistentStoreLoaderAddingCompletionHandler)completionHandler;

/*!
	\brief
	Asynchronously add the persistent store at the given URL.
	
	In case the store cannot be added, a recovery attempter offering to create a new store on top of the current one or simply retry will be available in the error.
	
	Will throw an exception if the persistent store coordinator is nil. Make sure to use the correct initializer.
	
	If no queue is specified, the completion handler block will be invoked on the main queue.
 */
- (void)addPersistentStoreAtLocation:(NSURL *)storeURL type:(NSString *)type configuration:(NSString *)configuration options:(NSDictionary *)options handlerQueue:(NSOperationQueue *)handlerQueue completionHandler:(EMBPersistentStoreLoaderAddingCompletionHandler)completionHandler;

/*!
	\brief
	The type of the completion handler called back when a store is removed.
 */
typedef void (^EMBPersistentStoreLoaderRemovingCompletionHandler)(BOOL removed, NSError *removingError);

/*!
	\brief
	Asynchronously remove the persistent store
	
	Will throw an exception if the persistent store coordinator is nil.
	
	If no queue is specified, the completion handler block will be invoked on the main queue.
 */
- (void)removePersistentStore:(NSPersistentStore *)persistentStore handlerQueue:(NSOperationQueue *)handlerQueue completionHandler:(EMBPersistentStoreLoaderRemovingCompletionHandler)completionHandler;

/*!
	\brief
	The type of the completion handler called back when a store is migrated.
 */
typedef void (^EMBPersistentStoreLoaderMigrationCompletionHandler)(NSPersistentStore *persistentStore, NSError *migrationError);

/*!
	\brief
	Asynchronously migrate the persistent store
	
	Will throw an exception if the persistent store coordinator is nil.
	
	If no queue is specified, the completion handler block will be invoked on the main queue.
 */
- (void)migratePersistentStore:(NSPersistentStore *)persistentStore toLocation:(NSURL *)location type:(NSString *)type options:(NSDictionary *)options handlerQueue:(NSOperationQueue *)handlerQueue completionHandler:(EMBPersistentStoreLoaderMigrationCompletionHandler)completionHandler;

/*!
	\brief
	In case you already have a persistent store coordinator and want to benefit
	from the asynchronous adding and removing of the loader, you can use these class methods.
 */
+ (void)addPersistentStoreAtLocation:(NSURL *)storeURL persistentStoreCoordinator:(NSPersistentStoreCoordinator *)persistentStoreCoordinator type:(NSString *)type configuration:(NSString *)configuration options:(NSDictionary *)options handlerQueue:(NSOperationQueue *)queue completionHandler:(EMBPersistentStoreLoaderAddingCompletionHandler)completionHandler;
+ (void)removePersistentStore:(NSPersistentStore *)persistentStore persistentStoreCoordinator:(NSPersistentStoreCoordinator *)persistentStoreCoordinator handlerQueue:(NSOperationQueue *)handlerQueue completionHandler:(EMBPersistentStoreLoaderRemovingCompletionHandler)completionHandler;
+ (void)migratePersistentStore:(NSPersistentStore *)persistentStore persistentStoreCoordinator:(NSPersistentStoreCoordinator *)persistentStoreCoordinator toLocation:(NSURL *)location type:(NSString *)type options:(NSDictionary *)options handlerQueue:(NSOperationQueue *)handlerQueue completionHandler:(EMBPersistentStoreLoaderMigrationCompletionHandler)completionHandler;

@end

/*!
	\brief
	This is kind of an implementation detail but it can be helpful for callers that want to
	programmatically handle the error containing a recovery attempter returned by the loader.
 */
typedef NS_ENUM(NSUInteger, EMBPersistentLoaderErrorRecoveryOptionIndex) {
	EMBPersistentLoaderErrorRecoveryOptionRetry = 0,
	EMBPersistentLoaderErrorRecoveryOptionNewStore = 1,
};
